# -*- coding: utf-8 -*-
from django.contrib import admin
from django.utils.translation import ugettext_lazy as _

from adminsortable.admin import SortableAdmin, SortableTabularInline

from .models import Campaign, Category, Keyword


class KeywordInline(SortableTabularInline):
    model = Keyword
    extra = 0


class CategoryAdmin(SortableAdmin):
    inlines = (KeywordInline, )


class CampaignAdmin(admin.ModelAdmin):
    list_display = ('subject', 'mc_title', 'send_time', 'hidden', 'category')
    search_fields = ('cid', 'subject', 'mc_title')
    list_filter = ('hidden', 'category')
    list_editable = ('hidden', )
    readonly_fields = ('cid', 'mc_title', 'subject', 'send_time', 'content_html', 'content_text', 'slug')

    _fieldsets = (
        (_('MailChimp Info'), {
            'fields': (
                'cid', 'mc_title', 'subject', 'send_time',
            )}
         ),
        (_('Visibility & Classification'), {
            'fields': (
                'hidden', 'category',
            )}
         ),
        (_('Content'), {
            'classes': ('collapse', ),
            'fields': (
                'content_text', 'content_html',
            )}
         ),
    )

    def get_fieldsets(self, request, obj=None):
        return self._fieldsets

    def has_add_permission(self, request):
        return False

    def has_delete_permission(self, request, obj=None):
        return False


admin.site.register(Category, CategoryAdmin)
admin.site.register(Campaign, CampaignAdmin)



# -*- coding: utf-8 -*-
from django.core.management.base import BaseCommand
from django.conf import settings
from django.utils.text import slugify

from pyrate.services import mailchimp

from ...models import Campaign, Category


class Command(BaseCommand):

    keywords = None

    @staticmethod
    def fetch_keywords():
        keyword_groups = {
            'name': {},
            'subject': {},
            'content': {},
        }

        for category in Category.objects.filter(smart_match=True):
            for kw in category.keyword_set.all():
                if kw.scope_name:
                    keyword_groups['name'][kw.value.lower()] = kw.category.id
                if kw.scope_subject:
                    keyword_groups['subject'][kw.value.lower()] = kw.category.id
                if kw.scope_content:
                    keyword_groups['content'][kw.value.lower()] = kw.category.id
        return keyword_groups

    def search_category(self, campaign):
        attr_list = (
            # kw-group, campaign-attr
            ('name', 'mc_title'),
            ('subject', 'subject'),
            ('content', 'content_text'),
        )

        category_id = None

        for kw_group, campaign_attr in attr_list:
            if not category_id:
                for kw, cat in self.keywords[kw_group].items():
                    if getattr(campaign, campaign_attr) and kw in getattr(campaign, campaign_attr).lower():
                        category_id = cat
                        break

        return category_id

    def handle(self, *args, **options):
        self.keywords = self.fetch_keywords()
        mc = mailchimp.MailchimpPyrate(settings.MAILCHIMP_API_KEY)
        response = mc.post('campaigns/list')
        for each in response['data']:
            campaign, created = Campaign.objects.get_or_create(cid=each['id'])
            campaign.send_time = each['send_time']
            campaign.mc_title = each['title']
            campaign.subject = each['subject']
            campaign.slug = slugify(each['subject'])[:50] if each['subject'] else campaign.pk

            # content
            try:
                response_content = mc.post('campaigns/content', content={'cid': campaign.cid})
            except Exception as e:
                print(e)
                campaign.hidden = True
            else:
                campaign.content_text = response_content['text']
                campaign.content_html = response_content['html']

            # match campaign to category (if not set yet)
            if not campaign.category:
                category_id = self.search_category(campaign)
                if category_id:
                    campaign.category = Category.objects.get(pk=category_id)

            campaign.save()

        print('imported %i campaigns' % len(response['data']))



# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding field 'Category.order'
        db.add_column(u'aldryn_mailchimp_category', 'order',
                      self.gf('django.db.models.fields.PositiveIntegerField')(default=1, db_index=True),
                      keep_default=False)

        # Adding field 'Keyword.order'
        db.add_column(u'aldryn_mailchimp_keyword', 'order',
                      self.gf('django.db.models.fields.PositiveIntegerField')(default=1, db_index=True),
                      keep_default=False)


        # Changing field 'Keyword.category'
        db.alter_column(u'aldryn_mailchimp_keyword', 'category_id', self.gf('adminsortable.fields.SortableForeignKey')(to=orm['aldryn_mailchimp.Category']))

    def backwards(self, orm):
        # Deleting field 'Category.order'
        db.delete_column(u'aldryn_mailchimp_category', 'order')

        # Deleting field 'Keyword.order'
        db.delete_column(u'aldryn_mailchimp_keyword', 'order')


        # Changing field 'Keyword.category'
        db.alter_column(u'aldryn_mailchimp_keyword', 'category_id', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['aldryn_mailchimp.Category']))

    models = {
        u'aldryn_mailchimp.campaign': {
            'Meta': {'ordering': "['-send_time']", 'object_name': 'Campaign'},
            'category': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['aldryn_mailchimp.Category']", 'null': 'True', 'blank': 'True'}),
            'cid': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'content_html': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'content_text': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'hidden': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'mc_title': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'send_time': ('django.db.models.fields.DateTimeField', [], {'null': 'True', 'blank': 'True'}),
            'slug': ('django.db.models.fields.SlugField', [], {'max_length': '50'}),
            'subject': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'})
        },
        u'aldryn_mailchimp.campaignarchiveplugin': {
            'Meta': {'object_name': 'CampaignArchivePlugin', '_ormbases': ['cms.CMSPlugin']},
            'categories': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'to': u"orm['aldryn_mailchimp.Category']", 'null': 'True', 'blank': 'True'}),
            u'cmsplugin_ptr': ('django.db.models.fields.related.OneToOneField', [], {'to': "orm['cms.CMSPlugin']", 'unique': 'True', 'primary_key': 'True'}),
            'count': ('django.db.models.fields.PositiveSmallIntegerField', [], {'null': 'True', 'blank': 'True'})
        },
        u'aldryn_mailchimp.category': {
            'Meta': {'ordering': "['order']", 'object_name': 'Category'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'order': ('django.db.models.fields.PositiveIntegerField', [], {'default': '1', 'db_index': 'True'}),
            'smart_match': ('django.db.models.fields.BooleanField', [], {'default': 'True'})
        },
        u'aldryn_mailchimp.keyword': {
            'Meta': {'ordering': "['order']", 'object_name': 'Keyword'},
            'category': ('adminsortable.fields.SortableForeignKey', [], {'to': u"orm['aldryn_mailchimp.Category']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'order': ('django.db.models.fields.PositiveIntegerField', [], {'default': '1', 'db_index': 'True'}),
            'scope_content': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'scope_name': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'scope_subject': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'value': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '255'})
        },
        u'aldryn_mailchimp.selectedcampaignsplugin': {
            'Meta': {'object_name': 'SelectedCampaignsPlugin', '_ormbases': ['cms.CMSPlugin']},
            'campaigns': ('django.db.models.fields.related.ManyToManyField', [], {'to': u"orm['aldryn_mailchimp.Campaign']", 'symmetrical': 'False'}),
            u'cmsplugin_ptr': ('django.db.models.fields.related.OneToOneField', [], {'to': "orm['cms.CMSPlugin']", 'unique': 'True', 'primary_key': 'True'})
        },
        u'aldryn_mailchimp.subscriptionplugin': {
            'Meta': {'object_name': 'SubscriptionPlugin', '_ormbases': ['cms.CMSPlugin']},
            'assign_language': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            u'cmsplugin_ptr': ('django.db.models.fields.related.OneToOneField', [], {'to': "orm['cms.CMSPlugin']", 'unique': 'True', 'primary_key': 'True'}),
            'list_id': ('django.db.models.fields.CharField', [], {'max_length': '20'})
        },
        'cms.cmsplugin': {
            'Meta': {'object_name': 'CMSPlugin'},
            'changed_date': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'creation_date': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'language': ('django.db.models.fields.CharField', [], {'max_length': '15', 'db_index': 'True'}),
            'level': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'}),
            'lft': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'}),
            'parent': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['cms.CMSPlugin']", 'null': 'True', 'blank': 'True'}),
            'placeholder': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['cms.Placeholder']", 'null': 'True'}),
            'plugin_type': ('django.db.models.fields.CharField', [], {'max_length': '50', 'db_index': 'True'}),
            'position': ('django.db.models.fields.PositiveSmallIntegerField', [], {'null': 'True', 'blank': 'True'}),
            'rght': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'}),
            'tree_id': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'})
        },
        'cms.placeholder': {
            'Meta': {'object_name': 'Placeholder'},
            'default_width': ('django.db.models.fields.PositiveSmallIntegerField', [], {'null': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'slot': ('django.db.models.fields.CharField', [], {'max_length': '50', 'db_index': 'True'})
        }
    }

    complete_apps = ['aldryn_mailchimp']


# -*- coding: utf-8 -*-
from django.core.urlresolvers import reverse
from django.db import models
from django.utils.translation import ugettext_lazy as _

from cms.models.pluginmodel import CMSPlugin
from adminsortable.models import Sortable
from adminsortable.fields import SortableForeignKey


class SubscriptionPlugin(CMSPlugin):

    list_id = models.CharField(_('List ID'), max_length=20)
    assign_language = models.BooleanField(
        _('Save user\'s language'), default=True, help_text=_('Save the user\'s language based on the page language'))

    def __unicode__(self):
        return unicode(self.list_id)


class CampaignManager(models.Manager):
    def published(self):
        return self.filter(send_time__isnull=False, hidden=False)


class Category(Sortable):
    name = models.CharField(_('name'), max_length=255)
    smart_match = models.BooleanField(
        _('Matching'), default=True, help_text=_('Match incoming campaigns to categories based on keywords')
    )

    class Meta(Sortable.Meta):
        ordering = ('order', 'name')
        verbose_name = _('Category')
        verbose_name_plural = _('Categories')

    def __unicode__(self):
        return unicode(self.name)


class Keyword(Sortable):
    value = models.CharField(_('value'), max_length=255, unique=True)
    category = SortableForeignKey(Category, verbose_name=_('category'))
    scope_name = models.BooleanField(_('search in campaign name'), default=True)
    scope_subject = models.BooleanField(_('search in campaign subject'), default=False)
    scope_content = models.BooleanField(_('search in campaign content'), default=False)

    class Meta(Sortable.Meta):
        ordering = ('order', 'value')
        verbose_name = _('Keyword')
        verbose_name_plural = _('Keywords')

    def __unicode__(self):
        return u'%s (%s)' % (self.value, self.category.name)


class Campaign(models.Model):
    cid = models.CharField(_('campaign id'), max_length=255, editable=False)
    mc_title = models.CharField(_('campaign title'), max_length=255, editable=False)
    subject = models.CharField(_('subject'), max_length=255, blank=True, null=True, editable=False)
    send_time = models.DateTimeField(_('time sent'), blank=True, null=True, editable=False)
    content_text = models.TextField(_('content text'), blank=True, null=True, editable=False)
    content_html = models.TextField(_('content HTML'), blank=True, null=True, editable=False)
    slug = models.SlugField(_('slug (generated)'), editable=False)
    hidden = models.BooleanField(_('hidden'), default=False)
    category = models.ForeignKey(Category, blank=True, null=True, help_text=_('leave empty to auto-match on import'))

    objects = CampaignManager()

    class Meta:
        ordering = ['-send_time']
        verbose_name = _('Campaign')
        verbose_name_plural = _('Campaigns')

    def __str__(self):
        return '%s (%s)' % (self.mc_title, self.subject)

    def get_absolute_url(self):
        return reverse('mailchimp_campaign_detail', kwargs={'pk': self.pk, 'slug': self.slug})


class CampaignArchivePlugin(CMSPlugin):
    count = models.PositiveSmallIntegerField(
        _('count'), null=True, blank=True, help_text=_('Leave blank to display all')
    )
    categories = models.ManyToManyField(
        Category, verbose_name=_('filter by category/categories'), blank=True, null=True
    )

    def copy_relations(self, old_instance):
        self.categories = old_instance.categories.all()


class SelectedCampaignsPlugin(CMSPlugin):
    campaigns = models.ManyToManyField(Campaign)

    def copy_relations(self, old_instance):
        self.campaigns = old_instance.campaigns.all()



# -*- coding: utf-8 -*-
from django.utils.translation import ugettext_lazy as _

from cms.app_base import CMSApp
from cms.apphook_pool import apphook_pool


class CampaignArchive(CMSApp):
    name = _('Campaign Archive')
    urls = ['aldryn_mailchimp.urls']


apphook_pool.register(CampaignArchive)
